/**
 * Common utilities and type definitions for the `Debug` module.
 *
 * @since 0.5.0
 */
import { IO } from 'fp-ts/lib/IO';
import { BehaviorSubject } from 'rxjs';
import { Cmd } from '../Cmd';
import { Dispatch } from '../Platform';
/**
 * @since 0.5.0
 */
export declare type Global = typeof window;
/**
 * @since 0.5.0
 */
export declare type DebugData<Model, Msg> = [DebugAction<Msg>, Model];
/**
 * @since 0.5.0
 */
export declare type DebugAction<Msg> = DebugInit | DebugMsg<Msg>;
/**
 * @since 0.5.0
 */
export interface DebugInit {
    type: 'INIT';
}
/**
 * Creates a `DebugInit`
 * @since 0.5.0
 */
export declare const debugInit: () => DebugInit;
/**
 * @since 0.5.0
 */
export interface DebugMsg<Msg> {
    type: 'MESSAGE';
    payload: Msg;
}
/**
 * Creates a `DebugMsg`
 * @since 0.5.0
 */
export declare const debugMsg: <Msg>(payload: Msg) => DebugMsg<Msg>;
/**
 * Extends `Msg` with a special kind of message from Debugger
 * @since 0.5.0
 */
export declare type MsgWithDebug<Model, Msg> = Msg | {
    type: '__DebugUpdateModel__';
    payload: Model;
} | {
    type: '__DebugApplyMsg__';
    payload: Msg;
};
/**
 * Defines a generic debugging function
 * @since 0.5.0
 */
export interface Debug<Model, Msg> {
    (data: DebugData<Model, Msg>): void;
}
/**
 * Defines a generic `Debugger`
 * @since 0.5.0
 */
export interface Debugger<Model, Msg> {
    (d: DebuggerR<Model, Msg>): Debug<Model, Msg>;
}
/**
 * Defines the dependencies for a `Debugger` function.
 * @since 0.5.0
 */
export interface DebuggerR<Model, Msg> {
    init: Model;
    debug$: BehaviorSubject<DebugData<Model, Msg>>;
    dispatch: Dispatch<MsgWithDebug<Model, Msg>>;
}
/**
 * Adds debugging capability to the provided `update` function.
 *
 * It tracks through the `debug$` stream every `Message` dispatched and resulting `Model` update.
 *
 * It also lets directly updating the application's state with a special `Message` of type:
 *
 * ```ts
 * {
 *   type: '__DebugUpdateModel__'
 *   payload: Model
 * }
 * ```
 *
 * or applying a message with:
 * ```ts
 * {
 *   type: '__DebugApplyMsg__';
 *   payload: Msg
 * }
 * ```
 * @since 0.5.3
 */
export declare function updateWithDebug<Model, Msg>(debug$: BehaviorSubject<DebugData<Model, Msg>>, update: (msg: Msg, model: Model) => [Model, Cmd<Msg>]): (msg: MsgWithDebug<Model, Msg>, model: Model) => [Model, Cmd<Msg>];
/**
 * Checks which type of debugger can be used (standard `console` or _Redux DevTool Extension_) based on provided `window` and prepares the subscription to the "debug" stream
 * @since 0.5.3
 */
export declare function runDebugger<Model, Msg>(win: Global): (deps: DebuggerR<Model, Msg>) => IO<void>;
