"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Common utilities and type definitions for the `Debug` module.
 *
 * @since 0.5.0
 */
var IO_1 = require("fp-ts/lib/IO");
var Option_1 = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var Cmd_1 = require("../Cmd");
var console_1 = require("./console");
var redux_devtool_1 = require("./redux-devtool");
/**
 * Creates a `DebugInit`
 * @since 0.5.0
 */
exports.debugInit = function () { return ({ type: 'INIT' }); };
/**
 * Creates a `DebugMsg`
 * @since 0.5.0
 */
exports.debugMsg = function (payload) { return ({ type: 'MESSAGE', payload: payload }); };
/**
 * Adds debugging capability to the provided `update` function.
 *
 * It tracks through the `debug$` stream every `Message` dispatched and resulting `Model` update.
 *
 * It also lets directly updating the application's state with a special `Message` of type:
 *
 * ```ts
 * {
 *   type: '__DebugUpdateModel__'
 *   payload: Model
 * }
 * ```
 *
 * or applying a message with:
 * ```ts
 * {
 *   type: '__DebugApplyMsg__';
 *   payload: Msg
 * }
 * ```
 * @since 0.5.3
 */
function updateWithDebug(debug$, update) {
    return function (msg, model) {
        if ('type' in msg) {
            switch (msg.type) {
                case '__DebugUpdateModel__':
                    return [msg.payload, Cmd_1.none];
                case '__DebugApplyMsg__':
                    return [update(msg.payload, model)[0], Cmd_1.none];
            }
        }
        var result = update(msg, model);
        debug$.next([exports.debugMsg(msg), result[0]]);
        return result;
    };
}
exports.updateWithDebug = updateWithDebug;
/**
 * Checks which type of debugger can be used (standard `console` or _Redux DevTool Extension_) based on provided `window` and prepares the subscription to the "debug" stream
 * @since 0.5.3
 */
function runDebugger(win) {
    return function (deps) {
        return pipeable_1.pipe(redux_devtool_1.getConnection(win), IO_1.map(Option_1.fold(function () { return console_1.consoleDebugger(); }, redux_devtool_1.reduxDevToolDebugger)), IO_1.chain(function (Debugger) { return function () { return deps.debug$.subscribe(Debugger(deps)); }; }));
    };
}
exports.runDebugger = runDebugger;
